import django

__title__ = "Django Compose Tag"
__version__ = "0.0.1"
__author__ = "Jérôme Bon"
__license__ = "BSD 3-Clause"
__copyright__ = "Copyright 2022 Hello Watt"

# Version synonym
VERSION = __version__

from .tag import composition_tag
